import { ref, onMounted, onUnmounted } from '@vue/composition-api';

function getQuery(min, max) {
    if (min === void 0) { min = 0; }
    if (max === void 0) { max = 0; }
    if (!min && !max) {
        throw new Error("Please specify at least one value, either 'min' or 'max'");
    }
    else if (min && !max) {
        return "only screen and (min-width: " + min + "px)";
    }
    else if (!min && max) {
        return "only screen and (max-width: " + (max - 1) + "px)";
    }
    else {
        return "only screen and (min-width: " + min + "px) and (max-width: " + (max - 1) + "px)";
    }
}

function useMedia(query, defaultState) {
    if (defaultState === void 0) { defaultState = false; }
    var mql;
    var matches = ref(defaultState);
    var updateMatchValue = function () { return (matches.value = mql.matches); };
    onMounted(function () {
        mql = window.matchMedia(query);
        mql.addListener(updateMatchValue);
        updateMatchValue();
    });
    onUnmounted(function () {
        mql.removeListener(updateMatchValue);
    });
    return matches;
}

function useMouse() {
    var docX = ref(0);
    var docY = ref(0);
    var mouseMoveHandler = function (e) {
        docX.value = e.pageX;
        docY.value = e.pageY;
    };
    onMounted(function () {
        document.addEventListener('mousemove', mouseMoveHandler);
    });
    onUnmounted(function () {
        document.removeEventListener('mousemove', mouseMoveHandler);
    });
    return {
        docX: docX,
        docY: docY
    };
}

function useMouseElement(element) {
    var docX = ref(0);
    var docY = ref(0);
    var elX = ref(0);
    var elY = ref(0);
    var elInfoX = ref(0);
    var elInfoY = ref(0);
    var elInfoW = ref(0);
    var elInfoH = ref(0);
    var mouseMoveHandler = function (e) {
        if (!element.value)
            return;
        docX.value = e.pageX;
        docY.value = e.pageY;
        var _a = element.value.getBoundingClientRect(), left = _a.left, top = _a.top, height = _a.height, width = _a.width;
        elInfoX.value = left + window.pageXOffset;
        elInfoY.value = top + window.pageYOffset;
        elInfoW.value = width;
        elInfoH.value = height;
        elX.value = e.pageX - elInfoX.value;
        elY.value = e.pageY - elInfoY.value;
    };
    onMounted(function () {
        document.addEventListener('mousemove', mouseMoveHandler);
    });
    onUnmounted(function () {
        document.removeEventListener('mousemove', mouseMoveHandler);
    });
    return {
        docX: docX,
        docY: docY,
        elX: elX,
        elY: elY,
        elInfoX: elInfoX,
        elInfoY: elInfoY,
        elInfoW: elInfoW,
        elInfoH: elInfoH
    };
}

function useHover(element) {
    var isHovered = ref(false);
    var mouseEnterHandler = function () {
        isHovered.value = true;
    };
    var mouseLeaveHandler = function () {
        isHovered.value = false;
    };
    onMounted(function () {
        if (!element.value)
            return;
        element.value.addEventListener('mouseenter', mouseEnterHandler);
        element.value.addEventListener('mouseleave', mouseLeaveHandler);
    });
    onUnmounted(function () {
        if (!element.value)
            return;
        element.value.removeEventListener('mouseenter', mouseEnterHandler);
        element.value.removeEventListener('mouseleave', mouseLeaveHandler);
    });
    return isHovered;
}

function useTimeoutFn(callback, ms, runOnMount) {
    if (ms === void 0) { ms = 0; }
    if (runOnMount === void 0) { runOnMount = true; }
    var isReady = ref(false);
    var isIdle = ref(!runOnMount);
    var timeout = null;
    var cancelTimer = function () {
        isReady.value = null;
        if (timeout) {
            clearTimeout(timeout);
            timeout = null;
        }
    };
    var setTimer = function () {
        isReady.value = false;
        if (isIdle.value)
            isIdle.value = false;
        if (timeout)
            clearTimeout(timeout);
        timeout = setTimeout(function () {
            isReady.value = true;
            callback();
            timeout = null;
        }, ms);
    };
    onMounted(function () {
        if (runOnMount)
            setTimer();
    });
    onUnmounted(cancelTimer);
    return { isReady: isReady, isIdle: isIdle, cancelTimer: cancelTimer, resetTimer: setTimer };
}

var noop = function () { return null; };
function useTimeout$$1(ms, runOnMount) {
    if (ms === void 0) { ms = 0; }
    if (runOnMount === void 0) { runOnMount = true; }
    var _a = useTimeoutFn(noop, ms, runOnMount), isReady = _a.isReady, isIdle = _a.isIdle, cancelTimer = _a.cancelTimer, resetTimer = _a.resetTimer;
    return { isReady: isReady, isIdle: isIdle, cancelTimer: cancelTimer, resetTimer: resetTimer };
}

export { getQuery, useMedia, useMouse, useMouseElement, useHover, useTimeoutFn, useTimeout$$1 as useTimeout };
